package Collectd::Plugins::Riemann;

require 5.008002;
use version;

=head1 NAME

Collectd::Plugins::Riemann - Collectd plugins for Riemann.

=head1 SYNOPSIS

To be used with L<Collectd>.

Please refer to the L<Collectd> documentation.
Or C<man collectd-perl>

=head1 FILES

/etc/collectd.conf
/etc/collectd.d/

=head1 SEE ALSO

Collectd, collectd-perl, collectd

=cut

our $VERSION = version->parse('v0.2.2');

1;

